@@ -26,26 +26,26 @@ Copyright (C) 2020  Martin Röbke
 
 """
 
-import json
+import argparse
 import io
 import itertools
+import json
 import logging
-from sys import stdin
-from typing import Iterable, Iterator, TypeVar, List, Optional
 from dataclasses import asdict
+from sys import stdin
+from typing import List, Optional
 
 from graphviz import Digraph, Graph
 from tdvisu.visualization_data import (VisualizationData, IncidenceGraphData,
                                        GeneralGraphData, SvgJoinData)
 from tdvisu.version import __date__, __version__ as version
 from tdvisu.svgjoin import svg_join
+from tdvisu.utilities import flatten, LOGLEVEL_EPILOG, logging_cfg
+from tdvisu.utilities import bag_node, solution_node, base_style
+from tdvisu.utilities import style_hide_edge, style_hide_node, emphasise_node
 
-logging.basicConfig(
-    format="%(asctime)s,%(msecs)d %(levelname)s"
-    "[%(filename)s:%(lineno)d] %(message)s",
-    datefmt='%Y-%m-%d %H:%M:%S')
 
-LOGGER = logging.getLogger(__name__)
+LOGGER = logging.getLogger('visualization.py')
 
 
 def read_json(json_data) -> dict:
@@ -75,20 +75,6 @@ def read_json(json_data) -> dict:
     return result
 
 
-_T = TypeVar('_T')
-
-
-def flatten(iterable: Iterable[Iterable[_T]]) -> Iterator[_T]:
-    """ Flatten at first level.
-
-    Turn ex=[[1,2],[3,4]] into
-    [1, 2, 3, 4]
-    and [ex,ex] into
-    [[1, 2], [3, 4], [1, 2], [3, 4]]
-    """
-    return itertools.chain.from_iterable(iterable)
-
-
 class Visualization:
     """Holds and processes the information needed to provide dot-format
     and image output for the visualization
@@ -105,150 +91,6 @@ class Visualization:
         # LOGGER.debug("self.__dict__:%s", self.__dict__)
         LOGGER.debug("self.data.svg_join:%s", self.data.svg_join)
 
-    @staticmethod
-    def base_style(graph, node, color='white', penwidth='1.0') -> None:
-        """Style the node with default fillcolor and penwidth."""
-        graph.node(node, fillcolor=color, penwidth=penwidth)
-
-    @staticmethod
-    def emphasise_node(graph, node, color='yellow',
-                       penwidth='2.5') -> None:
-        """Emphasise node with a different fillcolor (default:'yellow')
-        and penwidth (default:2.5).
-        """
-        if color:
-            graph.node(node, fillcolor=color)
-        if penwidth:
-            graph.node(node, penwidth=penwidth)
-
-    @staticmethod
-    def style_hide_node(graph, node) -> None:
-        """Make the node invisible during drawing."""
-        graph.node(node, style='invis')
-
-    @staticmethod
-    def style_hide_edge(graph, source, target) -> None:
-        """Make the edge source->target invisible during drawing."""
-        graph.edge(source, target, style='invis')
-
-    @staticmethod
-    def bag_node(head, tail, anchor='anchor', headcolor='white',
-                 tableborder=0, cellborder=0, cellspacing=0) -> str:
-        """HTML format with 'head' as the first label, then appending
-        further labels.
-
-        After the 'head' there is an (empty) anchor for edges with a name tag. e.g.
-        <TABLE BORDER="0" CELLBORDER="0" CELLSPACING="0">
-        <TR><TD BGCOLOR="white">bag 3</TD></TR><TR><TD PORT="anchor"></TD></TR>
-        <TR><TD>[1, 2, 5]</TD></TR><TR><TD>03/31/20 09:29:51</TD></TR>
-        <TR><TD>dtime=0.0051s</TD></TR></TABLE>
-        """
-        result = f"""<<TABLE BORDER=\"{tableborder}\" CELLBORDER=\"{cellborder}\"
-                  CELLSPACING=\"{cellspacing}\">
-                  <TR><TD BGCOLOR=\"{headcolor}\">{head}</TD></TR>
-                  <TR><TD PORT=\"{anchor}\"></TD></TR>"""
-
-        if isinstance(tail, str):
-            result += f"<TR><TD>{tail}</TD></TR>"
-        else:
-            for label in tail:
-                result += f"<TR><TD>{label}</TD></TR>"
-
-        result += "</TABLE>>"
-        return result
-
-    @staticmethod
-    def solution_node(
-            solution_table,
-            toplabel: str = '',
-            bottomlabel: str = '',
-            transpose: bool = False,
-            linesmax: int = 1000,
-            columnsmax: int = 50) -> str:
-        """Fill the node from the 2D 'solution_table' (columnbased!).
-        Optionally add a line above and/or below the table.
-
-        solution_table : 2D-arraylike, entries get converted to str
-
-        toplabel : string, placed above the table
-
-        bottomlabel : string, placed below the table
-
-        transpose : bool, whether to transpose the solution_table before
-        processing
-
-        linesmax : int, if positive it indicates the
-                maximum number of lines in the table to display.
-
-        columnsmax : int, if positive it indicates the
-                maximum number of columns to display + the last.
-
-        Example structure for four columns:
-        |----------|
-        | toplabel |
-        ------------
-        |v1|v2|v3|v4|
-        |0 |1 |0 |1 |
-        |1 |1 |0 |0 |
-        ...
-        ------------
-        | botlabel |
-        |----------|
-        """
-        result = ''
-        if toplabel:
-            result += toplabel + '|'
-
-        if len(solution_table) == 0:
-            result += 'empty'
-        else:
-            if transpose:
-                solution_table = list(zip(*solution_table))
-
-            # limit lines backwards from length of column
-            vslice = (min(-1, linesmax - len(solution_table[0]))
-                      if linesmax > 0 else -1)
-            # limit columns forwards minus one
-            hslice = (min(len(solution_table), columnsmax)
-                      if columnsmax > 0 else len(solution_table)) - 1
-
-            result += '{'                                       # insert table
-            for column in solution_table[:hslice]:
-                result += '{'                                   # start column
-                for row in column[:vslice]:
-                    result += str(row) + '|'
-                if vslice < -1:     # add one indicator of shortening
-                    result += '...' + '|'
-                for row in column[-1:]:
-                    result += str(row)
-                result += '}|'      # sep. between columns
-            # adding one column-skipping indicator
-            if hslice < len(solution_table) - 1:
-                result += '{'                                   # start column
-                for row in column[:vslice]:
-                    result += '...' + '|'
-                if vslice < -1:     # add one indicator of shortening
-                    result += '...' + '|'
-                for row in column[-1:]:
-                    result += '...'
-                result += '}|'      # sep. between columns
-            # last column (usually a summary of the previous cols)
-            for column in solution_table[-1:]:
-                result += '{'                                   # start column
-                for row in column[:vslice]:
-                    result += str(row) + '|'
-                if vslice < -1:     # add one indicator of shortening
-                    result += '...' + '|'
-                for row in column[-1:]:
-                    result += str(row)
-                result += '}'      # sep. between columns
-            result += '}'                                       # close table
-
-        if len(bottomlabel) > 0:
-            result += '|' + bottomlabel
-
-        return '{' + result + '}'
-
     def inspect_json(self, infile) -> VisualizationData:
         """Read and preprocess the needed data from the infile into VisualizationData."""
         LOGGER.debug("Reading from: %s", infile)
@@ -256,23 +98,24 @@ class Visualization:
         LOGGER.debug("Found keys: %s", visudata.keys())
 
         try:
-            incid = visudata['incidenceGraph']
-            general_graph = visudata['generalGraph']
-            svg_join = visudata.get('svg_join', None)
+            _incid = visudata['incidenceGraph']
+            _general_graph = visudata['generalGraph']
+            _svg_join = visudata.get('svg_join', None)
 
             incid_data: IncidenceGraphData = None
-            if incid:
-                incid['edges'] = [[x['id'], x['list']] for x in incid['edges']]
-                incid_data = IncidenceGraphData(**incid)
+            if _incid:
+                _incid['edges'] = [[x['id'], x['list']]
+                                   for x in _incid['edges']]
+                incid_data = IncidenceGraphData(**_incid)
             visudata.pop('incidenceGraph')
             general_graph_data: GeneralGraphData = None
-            if general_graph:
-                general_graph_data = GeneralGraphData(**general_graph)
+            if _general_graph:
+                general_graph_data = GeneralGraphData(**_general_graph)
             visudata.pop('generalGraph')
             svg_join_data: SvgJoinData = None
-            if svg_join:
-                svg_join_data = SvgJoinData(**svg_join)
-            if 'svg_join' in visudata.keys():
+            if _svg_join:
+                svg_join_data = SvgJoinData(**_svg_join)
+            if 'svg_join' in visudata:
                 visudata.pop('svg_join')
 
             self.timeline = visudata['tdTimeline']
@@ -317,7 +160,7 @@ class Visualization:
         for item in self.tree_dec['labeldict']:
             bagname = self.bagpre % str(item['id'])
             self.tree_dec_digraph.node(bagname,
-                                       self.bag_node(bagname, item['labels']))
+                                       bag_node(bagname, item['labels']))
 
         self.tree_dec_digraph.edges([(self.bagpre % str(first), self.bagpre % str(
             second)) for (first, second) in self.tree_dec['edgearray']])
@@ -332,7 +175,7 @@ class Visualization:
                 id_inv_bags = node[0]
                 if isinstance(id_inv_bags, int):
                     last_sol = solpre % id_inv_bags
-                    tdg.node(last_sol, self.solution_node(
+                    tdg.node(last_sol, solution_node(
                         *(node[1])), shape='record')
 
                     tdg.edge(self.bagpre % id_inv_bags, last_sol)
@@ -344,7 +187,7 @@ class Visualization:
 
                     id_inv_bags = tuple(id_inv_bags)
                     last_sol = soljoinpre % id_inv_bags
-                    tdg.node(last_sol, self.solution_node(
+                    tdg.node(last_sol, solution_node(
                         *(node[1])), shape='record')
 
                     tdg.edge(joinpre % id_inv_bags, last_sol)
@@ -383,29 +226,29 @@ class Visualization:
                         prevhead,
                         int) else joinpre %
                     tuple(prevhead))
-                self.base_style(tdg, bag)
+                base_style(tdg, bag)
                 if last_sol:
-                    self.style_hide_node(tdg, last_sol)
-                    self.style_hide_edge(tdg, bag, last_sol)
+                    style_hide_node(tdg, last_sol)
+                    style_hide_edge(tdg, bag, last_sol)
                     last_sol = ""
 
             if len(node) > 1:
                 # solution to be displayed
                 if isinstance(id_inv_bags, int):
                     last_sol = solpre % id_inv_bags
-                    self.emphasise_node(tdg, last_sol)
+                    emphasise_node(tdg, last_sol)
                     tdg.edge(self.bagpre % id_inv_bags, last_sol)
                 else:  # joined node with 2 bags
                     id_inv_bags = tuple(id_inv_bags)
                     last_sol = soljoinpre % id_inv_bags
-                    self.emphasise_node(tdg, last_sol)
+                    emphasise_node(tdg, last_sol)
 
-            self.emphasise_node(tdg,
-                                self.bagpre %
-                                id_inv_bags if isinstance(
-                                    id_inv_bags,
-                                    int) else joinpre %
-                                id_inv_bags)
+            emphasise_node(tdg,
+                           self.bagpre %
+                           id_inv_bags if isinstance(
+                               id_inv_bags,
+                               int) else joinpre %
+                           id_inv_bags)
             _filename = self.outfolder + self.data.td_file + '%d'
             tdg.render(
                 view=view, format='svg', filename=_filename %
@@ -592,8 +435,8 @@ class Visualization:
             # 5: Engine uses previous positions
             graph.engine = 'neato'
 
-        for (s, t) in edges:
-            graph.edge(vartag_n % s, vartag_n % t)
+        for (src, tar) in edges:
+            graph.edge(vartag_n % src, vartag_n % tar)
         for node in extra_nodes:
             graph.node(vartag_n % node)
 
@@ -733,13 +576,8 @@ class Visualization:
 
         # make edgelist variable-based (varX, clauseY), ...
         #  var_cl_iter [(1, 1), (4, 1), ...
-        vcmapping = map(
-            lambda y: map(
-                lambda x: (x, y[0]),
-                y[1]),
-            __incid.edges)
-
-        var_cl_iter = tuple(flatten(vcmapping))
+        var_cl_iter = tuple(flatten([[(x, y[0]) for x in y[1]]
+                                     for y in __incid.edges]))
 
         bodybaselen = len(g_incid.body)
         for i, variables in enumerate(timeline, start=1):    # all timesteps
@@ -796,25 +634,44 @@ class Visualization:
         """Analyzes content in data.svg_join for the call to svg_join."""
         sj_data = self.data.svg_join
         if not sj_data.base_names:
-            LOGGER.warn(
+            LOGGER.warning(
                 "svg_join data in JsonAPI contains no file-names to join.")
             return
         if isinstance(sj_data.base_names, str):
             sj_data.base_names = [sj_data.base_names]
         sj_data.num_images = int(sj_data.num_images)
         # Other arguments get handled directly in svgjoin for iterators etc.
-        svg_join(**asdict(sj_data))
+        # Use default outfolder only if folder is None
+        if sj_data.folder is None:
+            updated_sj_data = asdict(sj_data)
+            updated_sj_data["folder"] = self.outfolder
+            svg_join(**updated_sj_data)
+        else:
+            svg_join(**asdict(sj_data))
 
 
-def main(args):
-    """Main Function. Calling Visualization for arguments in 'args'."""
+def main(args: argparse.Namespace) -> None:
+    """
+    Main method running construct_dpdb_visu for arguments in 'args'
 
-    # get loglevel
-    try:
-        loglevel = int(float(args.loglevel))
-    except ValueError:
-        loglevel = args.loglevel.upper()
-    LOGGER.setLevel(loglevel)
+    Parameters
+    ----------
+    args : argparse.Namespace
+        The namespace containing all (command-line) parameters.
+
+    Returns
+    -------
+    None
+    """
+    loglevel = None  # passed the configuration of the root-logger
+    if args.loglevel is not None:
+        try:
+            loglevel = int(float(args.loglevel))
+        except ValueError:
+            loglevel = args.loglevel.upper()
+    logging.basicConfig(level=loglevel)  # Output logging for setup
+    logging_cfg(filename='logging.yml', loglevel=loglevel)
+    LOGGER.info("Called with '%s'", args)
 
     infile = args.infile
     outfolder = args.outfolder
@@ -830,8 +687,7 @@ def main(args):
 
 
 if __name__ == "__main__":
-
-    import argparse
+    # Parse args, call main
 
     PARSER = argparse.ArgumentParser(
         description="""
@@ -841,32 +697,22 @@ if __name__ == "__main__":
         under certain conditions; see COPYING for more information.
 
         Visualizing Dynamic Programming on Tree-Decompositions.""",
-        epilog="""Logging levels for python 3.8.2:
-            CRITICAL: 50
-            ERROR:    40
-            WARNING:  30
-            INFO:     20
-            DEBUG:    10
-            NOTSET:    0 (will traverse the logging hierarchy until a value is found)
-            """,
+        epilog=LOGLEVEL_EPILOG,
         formatter_class=argparse.RawDescriptionHelpFormatter)
 
     # possible to use stdin for the file.
     PARSER.add_argument('infile', nargs='?',
                         type=argparse.FileType('r', encoding='UTF-8'),
                         default=stdin,
-                        help="Input file for the visualization. "
-                        "Must be a Json fulfilling the 'JsonAPI_v1.3.md'")
+                        help="Input file for the visualization "
+                        "must be a Json fulfilling the 'JsonAPI.md'")
     PARSER.add_argument('outfolder',
                         help="Folder to output the visualization results.")
     PARSER.add_argument('--version', action='version',
                         version='%(prog)s ' + version + ', ' + __date__)
-    PARSER.add_argument(
-        '--loglevel',
-        default='WARNING',
-        help="set the minimal loglevel")
+    PARSER.add_argument('--loglevel', help="set the minimal loglevel for root")
 
     # get cmd-arguments
-    ARGS = PARSER.parse_args()
+    _args = PARSER.parse_args()
     # call main()
-    main(ARGS)
+    main(_args)
